@@ -80,6 +80,36 @@ static char sccsid[] = "@(#)ftp.c	5.38 (Berkeley) 4/22/91";
 
 #include "ftp_var.h"
 
+#ifdef USE_SOCKS
+
+#define CONNECT(A,B,C)  (socks_flag) ? Rconnect((A),(B),(B)) : \
+                                        connect((A),(B),(C))
+#define GETSOCKNAME(A,B,C) (socks_flag) ? Rgetsockname((A),(B),(C)) : \
+                                        getsockname((A),(B),(C))
+
+#define LISTEN(A,B)     (socks_flag) ? Rlisten((A),(B)) : \
+                                        listen((A),(B))
+#define ACCEPT(A,B,C)   (socks_flag) ? Raccept((A),(B),(C)) : \
+                                        accept((A),(B),(C))
+#if defined(SHORTENED_RBIND)
+#define bind            Rbind
+#else /* !SHORTENED_RBIND */
+/*
+ * Be sure to put in socks_bind_remoteAddr (32-bit integer) the IP
+ * address (in network byte order) of the host it is aiming for,
+ * which is typically the destination host of the previous connect().
+ */
+#define bind(a,b,c)     Rbind(a,b,c,socks_bind_remoteAddr)
+#endif /* SHORTENED_RBIND */
+#else /* !USE_SOCKS */
+#define CONNECT        connect
+#define GETSOCKNAME    getsockname
+#define LISTEN         listen
+#define ACCEPT         accept
+#define BIND           bind
+#endif /* USE_SOCKS */
+
+
 struct	sockaddr_in hisctladdr;
 struct	sockaddr_in data_addr;
 int	data = -1;
@@ -90,6 +120,10 @@ uid_t	getuid();
 extern 	void	lostpeer();
 off_t	restart_point = 0;
 
+#ifdef USE_SOCKS
+unsigned long socks_bind_remoteAddr;            /* Socks */
+#endif /* USE_SOCKS */
+
 extern char *strerror();
 extern int connected, errno;
 
@@ -131,7 +165,7 @@ hookup(host, port)
 		return (0);
 	}
 	hisctladdr.sin_port = port;
-	while (connect(s, (struct sockaddr *)&hisctladdr, sizeof (hisctladdr)) < 0) {
+	while (CONNECT(s, (struct sockaddr *)&hisctladdr, sizeof (hisctladdr)) < 0) {
 		if (hp && hp->h_addr_list[1]) {
 			int oerrno = errno;
 			extern char *inet_ntoa();
@@ -159,7 +193,10 @@ hookup(host, port)
 		goto bad;
 	}
 	len = sizeof (myctladdr);
-	if (getsockname(s, (struct sockaddr *)&myctladdr, &len) < 0) {
+#ifdef USE_SOCKS
+        socks_bind_remoteAddr = hisctladdr.sin_addr.s_addr;      /* Socks */
+#endif /* USE_SOCKS */
+	if (GETSOCKNAME(s, (struct sockaddr *)&myctladdr, &len) < 0) {
 		perror("ftp: getsockname");
 		code = -1;
 		goto bad;
@@ -1136,11 +1173,11 @@ noport:
 	    setsockopt(data, SOL_SOCKET, SO_DEBUG, (char *)&on, sizeof (on)) < 0)
 		perror("ftp: setsockopt (ignored)");
 	len = sizeof (data_addr);
-	if (getsockname(data, (struct sockaddr *)&data_addr, &len) < 0) {
+	if (GETSOCKNAME(data, (struct sockaddr *)&data_addr, &len) < 0) {
 		perror("ftp: getsockname");
 		goto bad;
 	}
-	if (listen(data, 1) < 0)
+	if (LISTEN(data, 1) < 0)
 		perror("ftp: listen");
 	if (sendport) {
 		a = (char *)&data_addr.sin_addr;
@@ -1179,7 +1216,7 @@ dataconn(lmode)
 	struct sockaddr_in from;
 	int s, fromlen = sizeof (from), tos;
 
-	s = accept(data, (struct sockaddr *) &from, &fromlen);
+	s = ACCEPT(data, (struct sockaddr *) &from, &fromlen);
 	if (s < 0) {
 		perror("ftp: accept");
 		(void) close(data), data = -1;
@@ -1195,7 +1232,7 @@ dataconn(lmode)
 
 #ifdef USE_SSL
 	ssl_data_active_flag=0;
-	if (ssl_encrypt_data) {
+	if (ssl_active_flag && ssl_encrypt_data) {
 	    /* do SSL */
 	    if (ssl_data_con!=NULL) {
 	      SSL_free(ssl_data_con);
