@@ -62,6 +62,22 @@ static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
  * FTP server.
  */
 
+#if defined(SOLARIS2) || defined(IRIX)
+#ifndef USE_STDARG
+#define USE_STDARG
+#endif /* USE_STDARG */
+#endif
+
+#include <stdio.h>
+
+#if !defined(_STDARG_H) && !defined(__STDARG_H__) && !defined(USE_STDARG)
+#include <varargs.h>
+#else
+#ifndef USE_STDARG
+#define USE_STDARG
+#endif
+#include <stdarg.h>
+#endif /* _STDARG_H || __STDARG_H__ */
 
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -98,7 +114,6 @@ static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
 #include <errno.h>
 #include <syslog.h>
 #include <unistd.h>
-#include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
@@ -107,12 +122,6 @@ static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
 #include <strings.h>
 #endif
 
-#if !defined(_STDARG_H) && !defined(__STDARG_H__)
-#include <varargs.h>
-#else
-#define USE_STDARG
-#endif /* _STDARG_H || __STDARG_H__ */
-
 #include "pathnames.h"
 
 #ifdef USE_SSL
@@ -149,8 +158,8 @@ static char *my_ssl_cert_file=NULL;
 #endif /* USE_SSL */
 
 #ifdef USE_STDARG
-int reply(int n, char *fmt, ...);
-int lreply(int n, char *fmt, ...);
+int reply(int n,...);
+int lreply(int n, ...);
 #else
 int reply();
 int lreply();
@@ -460,7 +469,7 @@ nextopt:
 	mode = MODE_S;
 	tmpline[0] = '\0';
 	(void) gethostname(hostname, sizeof (hostname));
-	reply(220, "%s FTP server (%s) ready.", hostname, version);
+	reply(220, "%s FTP server (%s) ready.", (char *)hostname, (char *)version);
 	(void) setjmp(errcatch);
 	for (;;)
 		(void) yyparse();
@@ -589,7 +598,7 @@ user(name)
 				break;
 		endusershell();
 		if (cp == NULL || checkuser(name)) {
-			reply(530, "User access denied.");
+			reply(530, "User access denied. shell=%s",shell);
 			if (logging)
 				syslog(LOG_NOTICE,
 				    "FTP LOGIN REFUSED FROM %s, %s",
@@ -1007,7 +1016,7 @@ dataconn(name, size, mode)
 	/* TODO XXXX fill in the blanks :-)
 	 */
 	ssl_data_active_flag=0;
-	if (ssl_encrypt_data) {
+	if (ssl_active_flag && ssl_encrypt_data) {
 	    /* do SSL */
 
 	    reply(150, "Opening %s mode SSL data connection for %s%s.",
@@ -1331,10 +1340,10 @@ fatal(s)
 	/* NOTREACHED */
 }
 
-/*VARARGS*/
 #ifdef USE_STDARG
-reply(int n, char *fmt, ...)
+reply(int n,...)
 #else
+/*VARARGS*/
 reply(va_alist)
 va_dcl
 #endif
@@ -1344,18 +1353,20 @@ va_dcl
 #endif /* USE_SSL */
 #ifdef USE_STDARG
 	va_list ap;
+	char *fmt;
 
-        va_start(ap,fmt);
+        va_start(ap,n);
+	fmt = va_arg(ap, char *);
 #else
 	va_list ap;
 	int n;
 	char *fmt;
 
         va_start(ap);
-#endif
 
 	n = va_arg(ap, int);
 	fmt = va_arg(ap, char *);
+#endif
 
 #ifdef USE_SSL
         /* assemble the output into a buffer */
@@ -1392,10 +1403,10 @@ va_dcl
 	va_end(ap);
 }
 
-/*VARARGS*/
 #ifdef USE_STDARG
-lreply(int n, char *fmt, ...)
+lreply(int n, ...)
 #else
+/*VARARGS*/
 lreply(va_alist)
 va_dcl
 #endif
@@ -1405,17 +1416,19 @@ va_dcl
 #endif /* USE_SSL */
 #ifdef USE_STDARG
 	va_list ap;
+	char *fmt;
 
         va_start(ap,fmt);
+	fmt = va_arg(ap, char *);
 #else
 	va_list ap;
 	int n;
 	char *fmt;
 
         va_start(ap);
-#endif
 	n = va_arg(ap, int);
 	fmt = va_arg(ap, char *);
+#endif
 
 #ifdef USE_SSL
         /* assemble the output into a buffer */
@@ -1897,13 +1910,26 @@ static FILE *fpShell;
 char *getusershell()
 {
   static char buf[BUFSIZ];
+  char *p;
+  int len;
 
   if (fpShell==NULL) {
     fpShell=fopen(SHELL_FILE,"r");
     if (fpShell==NULL)
 	return (NULL);
   }
-  return fgets(buf,BUFSIZ-1,fpShell);
+  p=fgets(buf,BUFSIZ-1,fpShell);
+
+  /* strip off the trailing newline ... --tjh */
+  if (p!=NULL) {
+    len=strlen(p);
+    if (len>0) {
+	if (p[len-1]=='\n')
+	    p[len-1]='\0';
+    }
+  }
+    
+  return p;
 }
 
 int endusershell()
